# Simple OAuth Redirect URI Wildcard

## Contents of this file

- [Introduction](#introduction)
- [Requirements](#requirements)
- [Installation](#installation)
- [Configuration](#configuration)
- [Usage](#usage)
- [Maintainers](#maintainers)

## Introduction

This module extends Drupal's Simple OAuth module to provide support for wildcard
patterns in OAuth2 redirect URIs. This is particularly useful for development
environments, preview deployments, or any scenario where you need to allow
dynamic subdomains in your OAuth2 redirect URIs.

### For what is this for?

When you want to allow dynamic redirect URIs in your OAuth2 clients, like
preview deployments or development environments with dynamic subdomains.

The idea is to allow wildcards in the redirect URIs of OAuth2 clients, so that
you don't have to add each subdomain manually. This is especially useful when
working with preview deployments, where each branch gets its own subdomain.

### Functionality

The following functionality is provided:

- Wildcard support in subdomains (e.g., `https://*.example.com`)
- Support for partial wildcards (e.g., `https://myapp-*.example.com`)
- Compatible with deployment platforms like Vercel
  (e.g., `https://myproject-git-*-username.vercel.app`)
- Secure validation to prevent misuse of wildcards in sensitive URI parts

Various validation rules are implemented to ensure security:

- Wildcards are only allowed in subdomain parts
- No wildcards in protocol, domain name, TLD or port numbers
- Original OAuth2 redirect URI validation is still applied

## Requirements

The module requires Drupal 10.3 or 11 and the following contrib modules:

- [Simple OAuth](https://www.drupal.org/project/simple_oauth)
- [Consumers](https://www.drupal.org/project/consumers)

## Installation

Install as you would normally install a contributed Drupal module. For further
information, see
[Installing Drupal Modules](https://www.drupal.org/docs/extending-drupal/installing-drupal-modules).

## Configuration

The module does not require any special configuration. Once enabled, it
automatically enhances the redirect URI validation of OAuth2 clients to
support wildcards.

## Usage

The module extends the redirect URI field in OAuth2 clients to support
wildcards.  
The following patterns are supported:

```yaml
# Full subdomain wildcard
https://*.example.com/callback

# Partial subdomain wildcard
https://myapp-*.example.com/callback

# Multiple subdomain levels
https://*.test.example.com/callback

# Deployment platform patterns
https://myproject-git-*-username.vercel.app/callback
```

Note that wildcards are only allowed in subdomain parts. They cannot be used in:

- Protocol (e.g., `*://example.com`)
- Domain name (e.g., `example.*.com`)
- TLD (e.g., `example.com.*`)
- Port numbers (e.g., `example.com:*`)

## Maintainers

Current maintainers:

- Christoph Niedermoser ([@nimoatwoodway](https://www.drupal.org/u/nimoatwoodway))
- Christian Foidl ([@chfoidl](https://www.drupal.org/u/chfoidl))
