<?php

declare(strict_types=1);

namespace Drupal\simple_oauth_redirect_uri_wildcard\EventSubscriber;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Url;
use Drupal\simple_oauth_redirect_uri_wildcard\KeyValueStore\AuthCodeKeyValueStore;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Handle kernel events.
 */
class KernelEventSubscriber implements EventSubscriberInterface {

  /**
   * Construct new instance.
   */
  public function __construct(
    protected readonly AuthCodeKeyValueStore $kv,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::RESPONSE => ['onResponse', 9999],
    ];
  }

  /**
   * Handle response.
   */
  public function onResponse(ResponseEvent $event): void {
    $request = $event->getRequest();
    $response = $event->getResponse();

    $authorizeUrl = Url::fromRoute('oauth2_token.authorize')->toString();

    // Only handle if path is oauth2 authorize.
    if ($request->getPathInfo() === $authorizeUrl && $response->headers->has('Location')) {
      $data = UrlHelper::parse($response->headers->get('Location'));
      $redirectUri = $request->get('redirect_uri');

      // Save the redirect uri when issuing auth code.
      // This is needed, because the OAuth 2.0 server does validate
      // the redirect uri of the client when the auth code is
      // exchanged for an access token.
      // The redirect uri is then not available in the request, so we
      // need to persist it somehow.
      if (
        $redirectUri &&
        array_key_exists('query', $data) &&
        array_key_exists('code', $data['query'])
      ) {
        $this->kv->set($data['query']['code'], $redirectUri);
      }
    }
  }

}
