<?php

declare(strict_types=1);

namespace Drupal\simple_oauth_redirect_uri_wildcard\KeyValueStore;

use Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface;
use Drupal\Core\KeyValueStore\KeyValueStoreExpirableInterface;
use Symfony\Component\DependencyInjection\Attribute\Autowire;

/**
 * Handle kv for auth_code <-> redirect_uri.
 */
class AuthCodeKeyValueStore {

  /**
   * The kv store.
   */
  protected KeyValueStoreExpirableInterface $kv;

  /**
   * Construct new instance.
   */
  public function __construct(
    #[Autowire(service: 'keyvalue.expirable')]
    readonly KeyValueExpirableFactoryInterface $factory,
    #[Autowire(param: 'simple_oauth_redirect_uri_wildcard.kv_ttl')]
    protected readonly int $ttl,
  ) {
    $this->kv = $factory->get('simple_oauth_redirect_uri_wildcard.auth_code_redirect_uris');
  }

  /**
   * Get saved redirect uri by auth code.
   *
   * The auth code is hashed via sha256, because
   * it would be too long for the kv key otherwise.
   */
  public function get(string $authCode): string|null {
    $key = hash('sha256', $authCode);

    return $this->kv->get($key);
  }

  /**
   * Save redirect uri by auth code.
   *
   * The auth code is hashed via sha256, because
   * it would be too long for the kv key otherwise.
   */
  public function set(string $authCode, string $redirectUri): string|null {
    $key = hash('sha256', $authCode);

    return $this->kv->setWithExpire($key, $redirectUri, $this->ttl);
  }

}
