<?php

declare(strict_types=1);

namespace Drupal\simple_oauth_redirect_uri_wildcard\Repository;

use Drupal\consumers\Negotiator;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Password\PasswordInterface;
use Drupal\simple_oauth\Entities\ClientEntityInterface;
use Drupal\simple_oauth\Repositories\ClientRepository as ClientRepositoryBase;
use Drupal\simple_oauth_redirect_uri_wildcard\Entity\ClientEntity;
use Drupal\simple_oauth_redirect_uri_wildcard\KeyValueStore\AuthCodeKeyValueStore;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Extends the core client repsository to supply custom client entity.
 */
class ClientRepository extends ClientRepositoryBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    PasswordInterface $passwordChecker,
    protected readonly Negotiator $negotiator,
    protected readonly RequestStack $requestStack,
    protected readonly AuthCodeKeyValueStore $authCodeKv,
    protected readonly LoggerInterface $logger,
  ) {
    parent::__construct($entityTypeManager, $passwordChecker);
  }

  /**
   * {@inheritdoc}
   */
  public function getClientEntity($clientIdentifier): ?ClientEntityInterface {
    $clientDrupalEntities = $this->entityTypeManager
      ->getStorage('consumer')
      ->loadByProperties(['client_id' => $clientIdentifier]);

    // Check if the client is registered.
    if (empty($clientDrupalEntities)) {
      return NULL;
    }

    /** @var \Drupal\consumers\Entity\Consumer $clientDrupalEntity */
    $clientDrupalEntity = reset($clientDrupalEntities);
    if (!$clientDrupalEntity) {
      return NULL;
    }

    $redirectUris = array_map(function ($uri) {
      return $uri['value'];
    }, $clientDrupalEntity->get('redirect')->getValue());

    return new ClientEntity(
        $clientDrupalEntity,
        $this->requestStack->getCurrentRequest(),
        $this->authCodeKv,
        $this->logger,
        $redirectUris,
    );
  }

}
